var fse = require('fs-extra');
var fs = require('fs');
var path = require('path');
var readdir = require('recursive-readdir');
var babel = require('babel-core');

function transformFile(filePath) {

    var result = babel.transformFileSync(filePath, {
        presets: _config.presets
    });

    fs.writeFile(filePath, result.code, function (err) {
        if (err) {
            throw err;
        }
    });
}

function compile() {

    if (!_srcPath) {
        console.log('[compile-es6] received a blank src path, please check again.');
        return;
    }

    if (!fs.existsSync(_srcPath)) {
        console.log('[compile-es6] file or folder not exists: ' + _srcPath);
        return;
    }

    // copy
    fse.copy(_srcPath, _targetPath, function (err) {
        
        // compile es6 to es
        readdir(_targetPath, function (err, files) {
            console.log(files);

            files.forEach(function (filepath) {
                if (_config.include.test(filepath)) {

                    if (!_config.exclude || !_config.exclude.test(filepath)) {
                        transformFile(filepath);
                    } else {
                        console.log('exclude: ', filepath);
                    }

                }
            });

            if (_done) {
                _done();
            } else {
                console.log('[compile-es6] compiles done.');
            }

        });
        
    });

}

var _srcPath;
var _targetPath;
var _config;
var _done;

module.exports = function (srcPath, targetPath, config, done) {

    if (arguments.length === 0) {
        console.log('[compile-es6] blank src path and target path.');
        return;
    }

    if (arguments.length === 1) {
        console.log('[compile-es6] blank target path.');
        return;
    }

    if (arguments.length === 2) {
        if (typeof arguments[0] !== 'string' || typeof arguments[1] !== 'string') {
            console.log('[compile-es6] src path and target path must be string.');
            return;
        } else {
            config = {
                include: /\.js$/,
                exclude: null,
                presets: ['es2015']
            };
        }
    }

    if (arguments.length === 3) {

        if (typeof arguments[0] !== 'string' || typeof arguments[1] !== 'string') {
            console.log('[compile-es6] src path and target path must be string.');
            return;
        } else {
            if (typeof arguments[2] === 'function') {
                done = config;
                config = {
                    include: /\.js$/,
                    exclude: null,
                    presets: ['es2015']
                };
            }
        }
    }

    _srcPath = srcPath;
    _targetPath = targetPath;
    _config = config;
    _done = done;

    console.log('[compile-es6] compiling...');
    compile();
};